/*
* file CcProjectFolder.java
*
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
* 
* com.ibm.rational.wvcm.stp.cc.CcProjectFolder
*
* (C) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
* Note to U.S. Government Users Restricted Rights:  Use, duplication or  
* disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
*/

package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.Feedback;
import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

/**
 * <p>A proxy for a ClearCase UCM project folder.
 * </p>
 * <p>Project folders are used to organize the projects in a project VOB. A project
 * folder may contain both UCM projects and project subfolders.
 * </p>
 * <p>A project VOB has one distinguished <i>root folder</i>.
 * </p>
 * <p>
 * For more project folder information, see the ClearCase "Guide to Developing Software with UCM"
 * manual, and the cleartool man page "mkfolder".
 * </p>
 */
public interface CcProjectFolder extends CcVobResource {

    /**
     * <p>
     * Create a new UCM project folder at the location specified by this proxy. The
     * location should be an object name selector specifying the folder's name
     * and the repository (project VOB) in which to create it.
     * </p>
     * <p>
     * Must specify the new project folder's parent folder by setting the
     * folder's {@link #PARENT_FOLDER} property.
     * </p>
     */
    CcProjectFolder doCreateCcProjectFolder(Feedback feedback) throws WvcmException;

    /** Is this the root folder of its project VOB? */
    PropertyName<Boolean> IS_ROOT = new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-root");

    /**
     * Get the value of this proxy's {@link #IS_ROOT} property.
     * 
     * @return true if this is the root project folder, else false.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    boolean getIsRoot() throws WvcmException;

    /** This folder's parent folder. Will be null if this is the root folder. */
    PropertyName<CcProjectFolder> PARENT_FOLDER =
        new PropertyName<CcProjectFolder>(PROPERTY_NAMESPACE,
                                          "parent-folder");

    /**
     * Get the value of this proxy's {@link #PARENT_FOLDER} property.
     * 
     * @return a client proxy for this folder's parent, or null if this is the
     *         root folder
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    CcProjectFolder getParentFolder() throws WvcmException;

    /**
     * Set the value of this proxy's {@link #PARENT_FOLDER} property.
     * @param parentFolder this folder's new parent folder; cannot be null.
     */
    void setParentFolder(CcProjectFolder parentFolder);

    /** This folder's subfolders. */
    PropertyName<ResourceList<CcProjectFolder>> SUBFOLDER_LIST =
        new PropertyName<ResourceList<CcProjectFolder>>(PROPERTY_NAMESPACE,
                                                        "subfolder-list");

    /**
     * Get the value of this proxy's {@link #SUBFOLDER_LIST}property.
     * 
     * @return a list of client proxies for this folder's subfolders
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    ResourceList<CcProjectFolder> getSubfolderList() throws WvcmException;

    /** This folder's projects. */
    PropertyName<ResourceList<CcProject>> PROJECT_LIST =
        new PropertyName<ResourceList<CcProject>>(PROPERTY_NAMESPACE,
                                                  "project-list");

    /**
     * Get the value of this proxy's {@link #PROJECT_LIST} property.
     * 
     * @return a list of client proxies for this folder's projects
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    ResourceList<CcProject> getProjectList() throws WvcmException;
}